@@ -1,5 +1,5 @@
 /******************************************************************************
-    Copyright (C) 2013 by Hugh Bailey <obs.jim@gmail.com>
+    Copyright (C) 2023 by Lain Bailey <lain@obsproject.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,23 +24,23 @@
 #include "graphics.h"
 
 struct gs_texture_render {
-	gs_texture_t  *target, *prev_target;
+	gs_texture_t *target, *prev_target;
 	gs_zstencil_t *zs, *prev_zs;
+	enum gs_color_space prev_space;
 
 	uint32_t cx, cy;
 
-	enum gs_color_format    format;
+	enum gs_color_format format;
 	enum gs_zstencil_format zsformat;
 
 	bool rendered;
 };
 
-gs_texrender_t *gs_texrender_create(enum gs_color_format format,
-		enum gs_zstencil_format zsformat)
+gs_texrender_t *gs_texrender_create(enum gs_color_format format, enum gs_zstencil_format zsformat)
 {
 	struct gs_texture_render *texrender;
 	texrender = bzalloc(sizeof(struct gs_texture_render));
-	texrender->format   = format;
+	texrender->format = format;
 	texrender->zsformat = zsformat;
 
 	return texrender;
@@ -55,8 +55,7 @@ void gs_texrender_destroy(gs_texrender_t *texrender)
 	}
 }
 
-static bool texrender_resetbuffer(gs_texrender_t *texrender, uint32_t cx,
-		uint32_t cy)
+static bool texrender_resetbuffer(gs_texrender_t *texrender, uint32_t cx, uint32_t cy)
 {
 	if (!texrender)
 		return false;
@@ -65,12 +64,11 @@ static bool texrender_resetbuffer(gs_texrender_t *texrender, uint32_t cx,
 	gs_zstencil_destroy(texrender->zs);
 
 	texrender->target = NULL;
-	texrender->zs     = NULL;
-	texrender->cx     = cx;
-	texrender->cy     = cy;
+	texrender->zs = NULL;
+	texrender->cx = cx;
+	texrender->cy = cy;
 
-	texrender->target = gs_texture_create(cx, cy, texrender->format,
-			1, NULL, GS_RENDER_TARGET);
+	texrender->target = gs_texture_create(cx, cy, texrender->format, 1, NULL, GS_RENDER_TARGET);
 	if (!texrender->target)
 		return false;
 
@@ -88,6 +86,11 @@ static bool texrender_resetbuffer(gs_texrender_t *texrender, uint32_t cx,
 }
 
 bool gs_texrender_begin(gs_texrender_t *texrender, uint32_t cx, uint32_t cy)
+{
+	return gs_texrender_begin_with_color_space(texrender, cx, cy, GS_CS_SRGB);
+}
+
+bool gs_texrender_begin_with_color_space(gs_texrender_t *texrender, uint32_t cx, uint32_t cy, enum gs_color_space space)
 {
 	if (!texrender || texrender->rendered)
 		return false;
@@ -108,8 +111,9 @@ bool gs_texrender_begin(gs_texrender_t *texrender, uint32_t cx, uint32_t cy)
 	gs_matrix_identity();
 
 	texrender->prev_target = gs_get_render_target();
-	texrender->prev_zs     = gs_get_zstencil_target();
-	gs_set_render_target(texrender->target, texrender->zs);
+	texrender->prev_zs = gs_get_zstencil_target();
+	texrender->prev_space = gs_get_color_space();
+	gs_set_render_target_with_color_space(texrender->target, texrender->zs, space);
 
 	gs_set_viewport(0, 0, texrender->cx, texrender->cy);
 
@@ -121,7 +125,7 @@ void gs_texrender_end(gs_texrender_t *texrender)
 	if (!texrender)
 		return;
 
-	gs_set_render_target(texrender->prev_target, texrender->prev_zs);
+	gs_set_render_target_with_color_space(texrender->prev_target, texrender->prev_zs, texrender->prev_space);
 
 	gs_matrix_pop();
 	gs_projection_pop();
@@ -140,3 +144,8 @@ gs_texture_t *gs_texrender_get_texture(const gs_texrender_t *texrender)
 {
 	return texrender ? texrender->target : NULL;
 }
+
+enum gs_color_format gs_texrender_get_format(const gs_texrender_t *texrender)
+{
+	return texrender->format;
+}
