@@ -1,5 +1,5 @@
 /******************************************************************************
-    Copyright (C) 2013 by Hugh Bailey <obs.jim@gmail.com>
+    Copyright (C) 2023 by Lain Bailey <lain@obsproject.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,22 +20,32 @@
 
 #include "../util/threading.h"
 #include "../util/darray.h"
-#include "../util/circlebuf.h"
+#include "../util/deque.h"
 #include "../util/platform.h"
 #include "../util/profiler.h"
+#include "../util/util_uint64.h"
 
 #include "audio-io.h"
 #include "audio-resampler.h"
 
+#ifdef _WIN32
+#define WIN32_LEAN_AND_MEAN
+#include <Windows.h>
+#include <avrt.h>
+#endif
+
 extern profiler_name_store_t *obs_get_profiler_name_store(void);
 
 /* #define DEBUG_AUDIO */
 
-#define nop() do {int invalid = 0;} while(0)
+#define nop()                    \
+	do {                     \
+		int invalid = 0; \
+	} while (0)
 
 struct audio_input {
 	struct audio_convert_info conversion;
-	audio_resampler_t         *resampler;
+	audio_resampler_t *resampler;
 
 	audio_output_callback_t callback;
 	void *param;
@@ -49,100 +59,52 @@ static inline void audio_input_free(struct audio_input *input)
 struct audio_mix {
 	DARRAY(struct audio_input) inputs;
 	float buffer[MAX_AUDIO_CHANNELS][AUDIO_OUTPUT_FRAMES];
+	float buffer_unclamped[MAX_AUDIO_CHANNELS][AUDIO_OUTPUT_FRAMES];
 };
 
 struct audio_output {
-	struct audio_output_info   info;
-	size_t                     block_size;
-	size_t                     channels;
-	size_t                     planes;
+	struct audio_output_info info;
+	size_t block_size;
+	size_t channels;
+	size_t planes;
 
-	pthread_t                  thread;
-	os_event_t                 *stop_event;
+	pthread_t thread;
+	os_event_t *stop_event;
 
-	bool                       initialized;
+	bool initialized;
 
-	audio_input_callback_t     input_cb;
-	void                       *input_param;
-	pthread_mutex_t            input_mutex;
-	struct audio_mix           mixes[MAX_AUDIO_MIXES];
+	audio_input_callback_t input_cb;
+	void *input_param;
+	pthread_mutex_t input_mutex;
+	struct audio_mix mixes[MAX_AUDIO_MIXES];
 };
 
 /* ------------------------------------------------------------------------- */
-/* the following functions are used to calculate frame offsets based upon
- * timestamps.  this will actually work accurately as long as you handle the
- * values correctly */
 
-static inline double ts_to_frames(const audio_t *audio, uint64_t ts)
-{
-	double audio_offset_d = (double)ts;
-	audio_offset_d /= 1000000000.0;
-	audio_offset_d *= (double)audio->info.samples_per_sec;
-
-	return audio_offset_d;
-}
-
-static inline double positive_round(double val)
-{
-	return floor(val+0.5);
-}
-
-static int64_t ts_diff_frames(const audio_t *audio, uint64_t ts1, uint64_t ts2)
-{
-	double diff = ts_to_frames(audio, ts1) - ts_to_frames(audio, ts2);
-	return (int64_t)positive_round(diff);
-}
-
-static int64_t ts_diff_bytes(const audio_t *audio, uint64_t ts1, uint64_t ts2)
-{
-	return ts_diff_frames(audio, ts1, ts2) * (int64_t)audio->block_size;
-}
-
-/* ------------------------------------------------------------------------- */
-
-static inline uint64_t min_uint64(uint64_t a, uint64_t b)
-{
-	return a < b ? a : b;
-}
-
-static inline size_t min_size(size_t a, size_t b)
-{
-	return a < b ? a : b;
-}
-
-#ifndef CLAMP
-#define CLAMP(val, minval, maxval) \
-	((val > maxval) ? maxval : ((val < minval) ? minval : val))
-#endif
-
-static bool resample_audio_output(struct audio_input *input,
-		struct audio_data *data)
+static bool resample_audio_output(struct audio_input *input, struct audio_data *data)
 {
 	bool success = true;
 
 	if (input->resampler) {
-		uint8_t  *output[MAX_AV_PLANES];
+		uint8_t *output[MAX_AV_PLANES];
 		uint32_t frames;
 		uint64_t offset;
 
 		memset(output, 0, sizeof(output));
 
-		success = audio_resampler_resample(input->resampler,
-				output, &frames, &offset,
-				(const uint8_t *const *)data->data,
-				data->frames);
+		success = audio_resampler_resample(input->resampler, output, &frames, &offset,
+						   (const uint8_t *const *)data->data, data->frames);
 
 		for (size_t i = 0; i < MAX_AV_PLANES; i++)
 			data->data[i] = output[i];
-		data->frames     = frames;
+		data->frames = frames;
 		data->timestamp -= offset;
 	}
 
 	return success;
 }
 
-static inline void do_audio_output(struct audio_output *audio,
-		size_t mix_idx, uint64_t timestamp, uint32_t frames)
+static inline void do_audio_output(struct audio_output *audio, size_t mix_idx, uint64_t timestamp, uint32_t frames)
 {
 	struct audio_mix *mix = &audio->mixes[mix_idx];
 	struct audio_data data;
@@ -150,10 +112,13 @@ static inline void do_audio_output(struct audio_output *audio,
 	pthread_mutex_lock(&audio->input_mutex);
 
 	for (size_t i = mix->inputs.num; i > 0; i--) {
-		struct audio_input *input = mix->inputs.array+(i-1);
+		struct audio_input *input = mix->inputs.array + (i - 1);
 
+		float(*buf)[AUDIO_OUTPUT_FRAMES] = input->conversion.allow_clipping ? mix->buffer_unclamped
+										    : mix->buffer;
 		for (size_t i = 0; i < audio->planes; i++)
-			data.data[i] = (uint8_t*)mix->buffer[i];
+			data.data[i] = (uint8_t *)buf[i];
+
 		data.frames = frames;
 		data.timestamp = timestamp;
 
@@ -178,10 +143,13 @@ static inline void clamp_audio_output(struct audio_output *audio, size_t bytes)
 		for (size_t plane = 0; plane < audio->planes; plane++) {
 			float *mix_data = mix->buffer[plane];
 			float *mix_end = &mix_data[float_size];
+			/* Unclamped mix is copied directly. */
+			memcpy(mix->buffer_unclamped[plane], mix_data, bytes);
 
 			while (mix_data < mix_end) {
 				float val = *mix_data;
-				val = (val >  1.0f) ?  1.0f : val;
+				val = (val == val) ? val : 0.0f;
+				val = (val > 1.0f) ? 1.0f : val;
 				val = (val < -1.0f) ? -1.0f : val;
 				*(mix_data++) = val;
 			}
@@ -189,8 +157,7 @@ static inline void clamp_audio_output(struct audio_output *audio, size_t bytes)
 	}
 }
 
-static void input_and_output(struct audio_output *audio,
-		uint64_t audio_time, uint64_t prev_time)
+static void input_and_output(struct audio_output *audio, uint64_t audio_time, uint64_t prev_time)
 {
 	size_t bytes = AUDIO_OUTPUT_FRAMES * audio->block_size;
 	struct audio_output_data data[MAX_AUDIO_MIXES];
@@ -201,8 +168,7 @@ static void input_and_output(struct audio_output *audio,
 	memset(data, 0, sizeof(data));
 
 #ifdef DEBUG_AUDIO
-	blog(LOG_DEBUG, "audio_time: %llu, prev_time: %llu, bytes: %lu",
-			audio_time, prev_time, bytes);
+	blog(LOG_DEBUG, "audio_time: %llu, prev_time: %llu, bytes: %lu", audio_time, prev_time, bytes);
 #endif
 
 	/* get mixers */
@@ -217,16 +183,14 @@ static void input_and_output(struct audio_output *audio,
 	for (size_t mix_idx = 0; mix_idx < MAX_AUDIO_MIXES; mix_idx++) {
 		struct audio_mix *mix = &audio->mixes[mix_idx];
 
-		memset(mix->buffer[0], 0, AUDIO_OUTPUT_FRAMES *
-				MAX_AUDIO_CHANNELS * sizeof(float));
+		memset(mix->buffer, 0, sizeof(mix->buffer));
 
 		for (size_t i = 0; i < audio->planes; i++)
 			data[mix_idx].data[i] = mix->buffer[i];
 	}
 
 	/* get new audio data */
-	success = audio->input_cb(audio->input_param, prev_time, audio_time,
-			&new_ts, active_mixes, data);
+	success = audio->input_cb(audio->input_param, prev_time, audio_time, &new_ts, active_mixes, data);
 	if (!success)
 		return;
 
@@ -240,56 +204,54 @@ static void input_and_output(struct audio_output *audio,
 
 static void *audio_thread(void *param)
 {
+#ifdef _WIN32
+	DWORD unused = 0;
+	const HANDLE handle = AvSetMmThreadCharacteristics(L"Audio", &unused);
+#endif
+
 	struct audio_output *audio = param;
 	size_t rate = audio->info.samples_per_sec;
 	uint64_t samples = 0;
 	uint64_t start_time = os_gettime_ns();
 	uint64_t prev_time = start_time;
-	uint64_t audio_time = prev_time;
-	uint32_t audio_wait_time =
-		(uint32_t)(audio_frames_to_ns(rate, AUDIO_OUTPUT_FRAMES) /
-				1000000);
 
 	os_set_thread_name("audio-io: audio thread");
 
 	const char *audio_thread_name =
-		profile_store_name(obs_get_profiler_name_store(),
-				"audio_thread(%s)", audio->info.name);
+		profile_store_name(obs_get_profiler_name_store(), "audio_thread(%s)", audio->info.name);
 
 	while (os_event_try(audio->stop_event) == EAGAIN) {
-		uint64_t cur_time;
+		samples += AUDIO_OUTPUT_FRAMES;
+		uint64_t audio_time = start_time + audio_frames_to_ns(rate, samples);
 
-		os_sleep_ms(audio_wait_time);
+		os_sleepto_ns_fast(audio_time);
 
 		profile_start(audio_thread_name);
 
-		cur_time = os_gettime_ns();
-		while (audio_time <= cur_time) {
-			samples += AUDIO_OUTPUT_FRAMES;
-			audio_time = start_time +
-				audio_frames_to_ns(rate, samples);
-
-			input_and_output(audio, audio_time, prev_time);
-			prev_time = audio_time;
-		}
+		input_and_output(audio, audio_time, prev_time);
+		prev_time = audio_time;
 
 		profile_end(audio_thread_name);
 
 		profile_reenable_thread();
 	}
 
+#ifdef _WIN32
+	if (handle)
+		AvRevertMmThreadCharacteristics(handle);
+#endif
+
 	return NULL;
 }
 
 /* ------------------------------------------------------------------------- */
 
-static size_t audio_get_input_idx(const audio_t *audio, size_t mix_idx,
-		audio_output_callback_t callback, void *param)
+static size_t audio_get_input_idx(const audio_t *audio, size_t mix_idx, audio_output_callback_t callback, void *param)
 {
 	const struct audio_mix *mix = &audio->mixes[mix_idx];
 
 	for (size_t i = 0; i < mix->inputs.num; i++) {
-		struct audio_input *input = mix->inputs.array+i;
+		struct audio_input *input = mix->inputs.array + i;
 
 		if (input->callback == callback && input->param == param)
 			return i;
@@ -298,28 +260,23 @@ static size_t audio_get_input_idx(const audio_t *audio, size_t mix_idx,
 	return DARRAY_INVALID;
 }
 
-static inline bool audio_input_init(struct audio_input *input,
-		struct audio_output *audio)
+static inline bool audio_input_init(struct audio_input *input, struct audio_output *audio)
 {
-	if (input->conversion.format          != audio->info.format          ||
+	if (input->conversion.format != audio->info.format ||
 	    input->conversion.samples_per_sec != audio->info.samples_per_sec ||
-	    input->conversion.speakers        != audio->info.speakers) {
-		struct resample_info from = {
-			.format          = audio->info.format,
-			.samples_per_sec = audio->info.samples_per_sec,
-			.speakers        = audio->info.speakers
-		};
+	    input->conversion.speakers != audio->info.speakers) {
+		struct resample_info from = {.format = audio->info.format,
+					     .samples_per_sec = audio->info.samples_per_sec,
+					     .speakers = audio->info.speakers};
 
-		struct resample_info to = {
-			.format          = input->conversion.format,
-			.samples_per_sec = input->conversion.samples_per_sec,
-			.speakers        = input->conversion.speakers
-		};
+		struct resample_info to = {.format = input->conversion.format,
+					   .samples_per_sec = input->conversion.samples_per_sec,
+					   .speakers = input->conversion.speakers};
 
 		input->resampler = audio_resampler_create(&to, &from);
 		if (!input->resampler) {
 			blog(LOG_ERROR, "audio_input_init: Failed to "
-			                "create resampler");
+					"create resampler");
 			return false;
 		}
 	} else {
@@ -329,29 +286,29 @@ static inline bool audio_input_init(struct audio_input *input,
 	return true;
 }
 
-bool audio_output_connect(audio_t *audio, size_t mi,
-		const struct audio_convert_info *conversion,
-		audio_output_callback_t callback, void *param)
+bool audio_output_connect(audio_t *audio, size_t mi, const struct audio_convert_info *conversion,
+			  audio_output_callback_t callback, void *param)
 {
 	bool success = false;
 
-	if (!audio || mi >= MAX_AUDIO_MIXES) return false;
+	if (!audio || mi >= MAX_AUDIO_MIXES)
+		return false;
 
 	pthread_mutex_lock(&audio->input_mutex);
 
 	if (audio_get_input_idx(audio, mi, callback, param) == DARRAY_INVALID) {
 		struct audio_mix *mix = &audio->mixes[mi];
-		struct audio_input input;
-		input.callback = callback;
-		input.param    = param;
+		struct audio_input input = {
+			.callback = callback,
+			.param = param,
+		};
 
 		if (conversion) {
 			input.conversion = *conversion;
 		} else {
 			input.conversion.format = audio->info.format;
 			input.conversion.speakers = audio->info.speakers;
-			input.conversion.samples_per_sec =
-				audio->info.samples_per_sec;
+			input.conversion.samples_per_sec = audio->info.samples_per_sec;
 		}
 
 		if (input.conversion.format == AUDIO_FORMAT_UNKNOWN)
@@ -359,8 +316,7 @@ bool audio_output_connect(audio_t *audio, size_t mi,
 		if (input.conversion.speakers == SPEAKERS_UNKNOWN)
 			input.conversion.speakers = audio->info.speakers;
 		if (input.conversion.samples_per_sec == 0)
-			input.conversion.samples_per_sec =
-				audio->info.samples_per_sec;
+			input.conversion.samples_per_sec = audio->info.samples_per_sec;
 
 		success = audio_input_init(&input, audio);
 		if (success)
@@ -372,17 +328,17 @@ bool audio_output_connect(audio_t *audio, size_t mi,
 	return success;
 }
 
-void audio_output_disconnect(audio_t *audio, size_t mix_idx,
-		audio_output_callback_t callback, void *param)
+void audio_output_disconnect(audio_t *audio, size_t mix_idx, audio_output_callback_t callback, void *param)
 {
-	if (!audio || mix_idx >= MAX_AUDIO_MIXES) return;
+	if (!audio || mix_idx >= MAX_AUDIO_MIXES)
+		return;
 
 	pthread_mutex_lock(&audio->input_mutex);
 
 	size_t idx = audio_get_input_idx(audio, mix_idx, callback, param);
 	if (idx != DARRAY_INVALID) {
 		struct audio_mix *mix = &audio->mixes[mix_idx];
-		audio_input_free(mix->inputs.array+idx);
+		audio_input_free(mix->inputs.array + idx);
 		da_erase(mix->inputs, idx);
 	}
 
@@ -391,14 +347,12 @@ void audio_output_disconnect(audio_t *audio, size_t mix_idx,
 
 static inline bool valid_audio_params(const struct audio_output_info *info)
 {
-	return info->format && info->name && info->samples_per_sec > 0 &&
-	       info->speakers > 0;
+	return info->format && info->name && info->samples_per_sec > 0 && info->speakers > 0;
 }
 
 int audio_output_open(audio_t **audio, struct audio_output_info *info)
 {
 	struct audio_output *out;
-	pthread_mutexattr_t attr;
 	bool planar = is_audio_planar(info->format);
 
 	if (!valid_audio_params(info))
@@ -406,32 +360,31 @@ int audio_output_open(audio_t **audio, struct audio_output_info *info)
 
 	out = bzalloc(sizeof(struct audio_output));
 	if (!out)
-		goto fail;
+		goto fail0;
 
 	memcpy(&out->info, info, sizeof(struct audio_output_info));
-	out->channels   = get_audio_channels(info->speakers);
-	out->planes     = planar ? out->channels : 1;
-	out->input_cb   = info->input_callback;
-	out->input_param= info->input_param;
-	out->block_size = (planar ? 1 : out->channels) *
-	                  get_audio_bytes_per_channel(info->format);
+	out->channels = get_audio_channels(info->speakers);
+	out->planes = planar ? out->channels : 1;
+	out->input_cb = info->input_callback;
+	out->input_param = info->input_param;
+	out->block_size = (planar ? 1 : out->channels) * get_audio_bytes_per_channel(info->format);
 
-	if (pthread_mutexattr_init(&attr) != 0)
-		goto fail;
-	if (pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE) != 0)
-		goto fail;
-	if (pthread_mutex_init(&out->input_mutex, &attr) != 0)
-		goto fail;
+	if (pthread_mutex_init_recursive(&out->input_mutex) != 0)
+		goto fail0;
 	if (os_event_init(&out->stop_event, OS_EVENT_TYPE_MANUAL) != 0)
-		goto fail;
+		goto fail1;
 	if (pthread_create(&out->thread, NULL, audio_thread, out) != 0)
-		goto fail;
+		goto fail2;
 
 	out->initialized = true;
 	*audio = out;
 	return AUDIO_OUTPUT_SUCCESS;
 
-fail:
+fail2:
+	os_event_destroy(out->stop_event);
+fail1:
+	pthread_mutex_destroy(&out->input_mutex);
+fail0:
 	audio_output_close(out);
 	return AUDIO_OUTPUT_FAIL;
 }
@@ -446,18 +399,18 @@ void audio_output_close(audio_t *audio)
 	if (audio->initialized) {
 		os_event_signal(audio->stop_event);
 		pthread_join(audio->thread, &thread_ret);
+		os_event_destroy(audio->stop_event);
+		pthread_mutex_destroy(&audio->input_mutex);
 	}
 
 	for (size_t mix_idx = 0; mix_idx < MAX_AUDIO_MIXES; mix_idx++) {
 		struct audio_mix *mix = &audio->mixes[mix_idx];
 
 		for (size_t i = 0; i < mix->inputs.num; i++)
-			audio_input_free(mix->inputs.array+i);
+			audio_input_free(mix->inputs.array + i);
 
 		da_free(mix->inputs);
 	}
-
-	os_event_destroy(audio->stop_event);
 	bfree(audio);
 }
 
@@ -468,7 +421,8 @@ const struct audio_output_info *audio_output_get_info(const audio_t *audio)
 
 bool audio_output_active(const audio_t *audio)
 {
-	if (!audio) return false;
+	if (!audio)
+		return false;
 
 	for (size_t mix_idx = 0; mix_idx < MAX_AUDIO_MIXES; mix_idx++) {
 		const struct audio_mix *mix = &audio->mixes[mix_idx];
@@ -482,20 +436,20 @@ bool audio_output_active(const audio_t *audio)
 
 size_t audio_output_get_block_size(const audio_t *audio)
 {
-	return audio ? audio->block_size : 0;
+	return audio->block_size;
 }
 
 size_t audio_output_get_planes(const audio_t *audio)
 {
-	return audio ? audio->planes : 0;
+	return audio->planes;
 }
 
 size_t audio_output_get_channels(const audio_t *audio)
 {
-	return audio ? audio->channels : 0;
+	return audio->channels;
 }
 
 uint32_t audio_output_get_sample_rate(const audio_t *audio)
 {
-	return audio ? audio->info.samples_per_sec : 0;
+	return audio->info.samples_per_sec;
 }
