@@ -113,6 +113,8 @@ struct obs_module {
 	void *module;
 	bool loaded;
 
+	enum obs_module_load_state load_state;
+
 	bool (*load)(void);
 	void (*unload)(void);
 	void (*post_load)(void);
@@ -125,7 +127,28 @@ struct obs_module {
 	const char *(*description)(void);
 	const char *(*author)(void);
 
+	struct obs_module_metadata *metadata;
+
 	struct obs_module *next;
+
+	DARRAY(char *) sources;
+	DARRAY(char *) outputs;
+	DARRAY(char *) encoders;
+	DARRAY(char *) services;
+};
+
+struct obs_disabled_module {
+	char *mod_name;
+
+	enum obs_module_load_state load_state;
+
+	struct obs_module_metadata *metadata;
+	struct obs_disabled_module *next;
+
+	DARRAY(char *) sources;
+	DARRAY(char *) outputs;
+	DARRAY(char *) encoders;
+	DARRAY(char *) services;
 };
 
 extern void free_module(struct obs_module *mod);
@@ -143,6 +166,37 @@ static inline void free_module_path(struct obs_module_path *omp)
 	}
 }
 
+struct obs_module_metadata {
+	char *display_name;
+	char *version;
+	char *id;
+	char *os_arch;
+	char *description;
+	char *long_description;
+	bool has_icon;
+	bool has_banner;
+	char *repository_url;
+	char *support_url;
+	char *website_url;
+	char *name;
+};
+
+static inline void free_module_metadata(struct obs_module_metadata *omi)
+{
+	if (omi) {
+		bfree(omi->display_name);
+		bfree(omi->version);
+		bfree(omi->id);
+		bfree(omi->os_arch);
+		bfree(omi->description);
+		bfree(omi->long_description);
+		bfree(omi->repository_url);
+		bfree(omi->support_url);
+		bfree(omi->website_url);
+		bfree(omi->name);
+	}
+}
+
 static inline bool check_path(const char *data, const char *path, struct dstr *output)
 {
 	dstr_copy(output, path);
@@ -405,6 +459,9 @@ struct obs_core_audio {
 
 	pthread_mutex_t task_mutex;
 	struct deque tasks;
+
+	volatile bool prevent_monitoring_duplication;
+	struct obs_source *monitoring_duplicating_source;
 };
 
 /* user sources, output channels, and displays */
@@ -487,8 +544,12 @@ typedef DARRAY(struct obs_source_info) obs_source_info_array_t;
 
 struct obs_core {
 	struct obs_module *first_module;
+	struct obs_module *first_disabled_module;
+
 	DARRAY(struct obs_module_path) module_paths;
 	DARRAY(char *) safe_modules;
+	DARRAY(char *) disabled_modules;
+	DARRAY(char *) core_modules;
 
 	obs_source_info_array_t source_types;
 	obs_source_info_array_t input_types;
@@ -824,6 +885,8 @@ struct obs_source {
 	int64_t sync_offset;
 	int64_t last_sync_offset;
 	float balance;
+	/* audio_is_duplicated: tracks whether a source appears multiple times in the audio tree during this tick */
+	bool audio_is_duplicated;
 
 	/* async video data */
 	gs_texture_t *async_textures[MAX_AV_PLANES];
@@ -1165,6 +1228,7 @@ struct obs_output {
 	os_event_t *stopping_event;
 	pthread_mutex_t interleaved_mutex;
 	DARRAY(struct encoder_packet) interleaved_packets;
+	size_t interleaver_max_batch_size;
 	int stop_code;
 
 	int reconnect_retry_sec;
