@@ -1401,6 +1401,14 @@ void obs_shutdown(void)
 	}
 	obs->first_module = NULL;
 
+	module = obs->first_disabled_module;
+	while (module) {
+		struct obs_module *next = module->next;
+		free_module(module);
+		module = next;
+	}
+	obs->first_disabled_module = NULL;
+
 	obs_free_data();
 	obs_free_audio();
 	obs_free_video();
@@ -1412,14 +1420,26 @@ void obs_shutdown(void)
 	obs->procs = NULL;
 	obs->signals = NULL;
 
-	for (size_t i = 0; i < obs->module_paths.num; i++)
+	for (size_t i = 0; i < obs->module_paths.num; i++) {
 		free_module_path(obs->module_paths.array + i);
+	}
 	da_free(obs->module_paths);
 
-	for (size_t i = 0; i < obs->safe_modules.num; i++)
+	for (size_t i = 0; i < obs->safe_modules.num; i++) {
 		bfree(obs->safe_modules.array[i]);
+	}
 	da_free(obs->safe_modules);
 
+	for (size_t i = 0; i < obs->disabled_modules.num; i++) {
+		bfree(obs->disabled_modules.array[i]);
+	}
+	da_free(obs->disabled_modules);
+
+	for (size_t i = 0; i < obs->core_modules.num; i++) {
+		bfree(obs->core_modules.array[i]);
+	}
+	da_free(obs->core_modules);
+
 	if (obs->name_store_owned)
 		profiler_name_store_free(obs->name_store);
 
@@ -1832,7 +1852,7 @@ void obs_canvas_enum_scenes(obs_canvas_t *canvas, bool (*enum_proc)(void *, obs_
 	while (source) {
 		obs_source_t *s = obs_source_get_ref(source);
 		if (s) {
-			if (obs_source_is_scene(source) && !enum_proc(param, s)) {
+			if (source->info.type == OBS_SOURCE_TYPE_SCENE && !enum_proc(param, s)) {
 				obs_source_release(s);
 				break;
 			}
@@ -2238,7 +2258,7 @@ static obs_source_t *obs_load_source_type(obs_data_t *source_data, bool is_priva
 	if (!*v_id)
 		v_id = id;
 
-	if (strcmp(id, scene_info.id) == 0 || strcmp(id, group_info.id) == 0) {
+	if (obs_source_type_is_scene(id) || obs_source_type_is_group(id)) {
 		const char *canvas_uuid = obs_data_get_string(source_data, "canvas_uuid");
 		canvas = obs_get_canvas_by_uuid(canvas_uuid);
 		/* Fall back to main canvas if canvas cannot be found. */
@@ -3047,18 +3067,22 @@ void start_raw_video(video_t *v, const struct video_scale_info *conversion, uint
 		     void (*callback)(void *param, struct video_data *frame), void *param)
 {
 	struct obs_core_video_mix *video = get_mix_for_video(v);
-	if (!video)
-		return;
-	if (video_output_connect2(v, conversion, frame_rate_divisor, callback, param))
+
+	// TODO: Make affected outputs use views/canvasses, and revert this later.
+	// https://github.com/obsproject/obs-studio/pull/12379
+	// https://github.com/obsproject/obs-studio/issues/12366
+	if (video_output_connect2(v, conversion, frame_rate_divisor, callback, param) && video)
 		os_atomic_inc_long(&video->raw_active);
 }
 
 void stop_raw_video(video_t *v, void (*callback)(void *param, struct video_data *frame), void *param)
 {
 	struct obs_core_video_mix *video = get_mix_for_video(v);
-	if (!video)
-		return;
-	if (video_output_disconnect2(v, callback, param))
+
+	// TODO: Make affected outputs use views/canvasses, and revert this later.
+	// https://github.com/obsproject/obs-studio/pull/12379
+	// https://github.com/obsproject/obs-studio/issues/12366
+	if (video_output_disconnect2(v, callback, param) && video)
 		os_atomic_dec_long(&video->raw_active);
 }
 
