@@ -15,14 +15,28 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 #include <obs-module.h>
+#include <util/platform.h>
 
 OBS_DECLARE_MODULE()
 OBS_MODULE_USE_DEFAULT_LOCALE("linux-v4l2", "en-US")
+MODULE_EXPORT const char *obs_module_description(void)
+{
+	return "Video4Linux2(V4L2) sources/virtual camera";
+}
 
 extern struct obs_source_info v4l2_input;
+extern struct obs_output_info virtualcam_info;
+extern bool loopback_module_available();
 
 bool obs_module_load(void)
 {
 	obs_register_source(&v4l2_input);
+
+	if (loopback_module_available()) {
+		obs_register_output(&virtualcam_info);
+	} else {
+		blog(LOG_WARNING, "v4l2loopback not installed, virtual camera not registered");
+	}
+
 	return true;
 }
