@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015 Hugh Bailey <obs.jim@gmail.com>
+ * Copyright (c) 2023 Lain Bailey <lain@obsproject.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -16,22 +16,24 @@
 
 #pragma once
 
+#include <stdbool.h>
 #include <stdint.h>
 
 enum ffm_packet_type {
 	FFM_PACKET_VIDEO,
-	FFM_PACKET_AUDIO
+	FFM_PACKET_AUDIO,
+	FFM_PACKET_CHANGE_FILE,
 };
 
-#define FFM_SUCCESS      0
-#define FFM_ERROR       -1
+#define FFM_SUCCESS 0
+#define FFM_ERROR -1
 #define FFM_UNSUPPORTED -2
 
 struct ffm_packet_info {
-	int64_t              pts;
-	int64_t              dts;
-	uint32_t             size;
-	uint32_t             index;
+	int64_t pts;
+	int64_t dts;
+	uint32_t size;
+	uint32_t index;
 	enum ffm_packet_type type;
-	bool                 keyframe;
+	bool keyframe;
 };
