@@ -16,6 +16,7 @@ extern struct obs_output_info rtmp_output_info;
 extern struct obs_output_info null_output_info;
 extern struct obs_output_info flv_output_info;
 extern struct obs_output_info mp4_output_info;
+extern struct obs_output_info mov_output_info;
 
 #if defined(_WIN32) && defined(MBEDTLS_THREADING_ALT)
 void mbed_mutex_init(mbedtls_threading_mutex_t *m)
@@ -63,6 +64,7 @@ bool obs_module_load(void)
 	obs_register_output(&null_output_info);
 	obs_register_output(&flv_output_info);
 	obs_register_output(&mp4_output_info);
+	obs_register_output(&mov_output_info);
 	return true;
 }
 
