@@ -3,29 +3,30 @@
 #include <util/util.hpp>
 #include <obs-module.h>
 #include <sys/stat.h>
-#include <windows.h>
+#include <combaseapi.h>
 #include <gdiplus.h>
 #include <algorithm>
 #include <string>
 #include <memory>
+#include <locale>
 
 using namespace std;
 using namespace Gdiplus;
 
-#define warning(format, ...) blog(LOG_WARNING, "[%s] " format, \
-		obs_source_get_name(source), ##__VA_ARGS__)
+#define warning(format, ...) blog(LOG_WARNING, "[%s] " format, obs_source_get_name(source), ##__VA_ARGS__)
 
-#define warn_stat(call) \
-	do { \
-		if (stat != Ok) \
-			warning("%s: %s failed (%d)", __FUNCTION__, call, \
-					(int)stat); \
+#define warn_stat(call)                                                               \
+	do {                                                                          \
+		if (stat != Ok)                                                       \
+			warning("%s: %s failed (%d)", __FUNCTION__, call, (int)stat); \
 	} while (false)
 
 #ifndef clamp
 #define clamp(val, min_val, max_val) \
-	if (val < min_val) val = min_val; \
-	else if (val > max_val) val = max_val;
+	if (val < min_val)           \
+		val = min_val;       \
+	else if (val > max_val)      \
+		val = max_val;
 #endif
 
 #define MIN_SIZE_CX 2
@@ -37,6 +38,8 @@ using namespace Gdiplus;
 
 /* ------------------------------------------------------------------------- */
 
+/* clang-format off */
+
 #define S_FONT                          "font"
 #define S_USE_FILE                      "read_from_file"
 #define S_FILE                          "file"
@@ -62,6 +65,8 @@ using namespace Gdiplus;
 #define S_EXTENTS_WRAP                  "extents_wrap"
 #define S_EXTENTS_CX                    "extents_cx"
 #define S_EXTENTS_CY                    "extents_cy"
+#define S_TRANSFORM                     "transform"
+#define S_ANTIALIASING                  "antialiasing"
 
 #define S_ALIGN_LEFT                    "left"
 #define S_ALIGN_CENTER                  "center"
@@ -71,6 +76,11 @@ using namespace Gdiplus;
 #define S_VALIGN_CENTER                 S_ALIGN_CENTER
 #define S_VALIGN_BOTTOM                 "bottom"
 
+#define S_TRANSFORM_NONE                0
+#define S_TRANSFORM_UPPERCASE           1
+#define S_TRANSFORM_LOWERCASE           2
+#define S_TRANSFORM_STARTCASE           3
+
 #define T_(v)                           obs_module_text(v)
 #define T_FONT                          T_("Font")
 #define T_USE_FILE                      T_("ReadFromFile")
@@ -97,6 +107,8 @@ using namespace Gdiplus;
 #define T_EXTENTS_WRAP                  T_("UseCustomExtents.Wrap")
 #define T_EXTENTS_CX                    T_("Width")
 #define T_EXTENTS_CY                    T_("Height")
+#define T_TRANSFORM                     T_("Transform")
+#define T_ANTIALIASING                  T_("Antialiasing")
 
 #define T_FILTER_TEXT_FILES             T_("Filter.TextFiles")
 #define T_FILTER_ALL_FILES              T_("Filter.AllFiles")
@@ -109,6 +121,13 @@ using namespace Gdiplus;
 #define T_VALIGN_CENTER                 T_ALIGN_CENTER
 #define T_VALIGN_BOTTOM                 T_("VerticalAlignment.Bottom")
 
+#define T_TRANSFORM_NONE                T_("Transform.None")
+#define T_TRANSFORM_UPPERCASE           T_("Transform.Uppercase")
+#define T_TRANSFORM_LOWERCASE           T_("Transform.Lowercase")
+#define T_TRANSFORM_STARTCASE           T_("Transform.Startcase")
+
+/* clang-format on */
+
 /* ------------------------------------------------------------------------- */
 
 static inline DWORD get_alpha_val(uint32_t opacity)
@@ -145,7 +164,8 @@ template<typename T, typename T2, BOOL WINAPI deleter(T2)> class GDIObj {
 
 	inline GDIObj &Replace(T obj_)
 	{
-		if (obj) deleter(obj);
+		if (obj)
+			deleter(obj);
 		obj = obj_;
 		return *this;
 	}
@@ -153,14 +173,18 @@ template<typename T, typename T2, BOOL WINAPI deleter(T2)> class GDIObj {
 public:
 	inline GDIObj() {}
 	inline GDIObj(T obj_) : obj(obj_) {}
-	inline ~GDIObj() {deleter(obj);}
+	inline ~GDIObj() { deleter(obj); }
 
-	inline T operator=(T obj_) {Replace(obj_); return obj;}
+	inline T operator=(T obj_)
+	{
+		Replace(obj_);
+		return obj;
+	}
 
-	inline operator T() const {return obj;}
+	inline operator T() const { return obj; }
 
-	inline bool operator==(T obj_) const {return obj == obj_;}
-	inline bool operator!=(T obj_) const {return obj != obj_;}
+	inline bool operator==(T obj_) const { return obj == obj_; }
+	inline bool operator!=(T obj_) const { return obj != obj_; }
 };
 
 using HDCObj = GDIObj<HDC, HDC, DeleteDC>;
@@ -172,13 +196,13 @@ using HBITMAPObj = GDIObj<HBITMAP, HGDIOBJ, DeleteObject>;
 enum class Align {
 	Left,
 	Center,
-	Right
+	Right,
 };
 
 enum class VAlign {
 	Top,
 	Center,
-	Bottom
+	Bottom,
 };
 
 struct TextSource {
@@ -217,6 +241,7 @@ struct TextSource {
 	bool italic = false;
 	bool underline = false;
 	bool strikeout = false;
+	bool antialiasing = true;
 	bool vertical = false;
 
 	bool use_outline = false;
@@ -225,19 +250,23 @@ struct TextSource {
 	uint32_t outline_opacity = 100;
 
 	bool use_extents = false;
+	bool old_extents = false;
 	bool wrap = false;
 	uint32_t extents_cx = 0;
 	uint32_t extents_cy = 0;
 
+	int text_transform = S_TRANSFORM_NONE;
+
 	bool chatlog_mode = false;
 	int chatlog_lines = 6;
 
 	/* --------------------------- */
 
-	inline TextSource(obs_source_t *source_, obs_data_t *settings) :
-		source                (source_),
-		hdc                   (CreateCompatibleDC(nullptr)),
-		graphics              (hdc)
+	inline TextSource(obs_source_t *source_, obs_data_t *settings, bool old_extents_)
+		: source(source_),
+		  hdc(CreateCompatibleDC(nullptr)),
+		  graphics(hdc),
+		  old_extents(old_extents_)
 	{
 		obs_source_update(source, settings);
 	}
@@ -254,19 +283,18 @@ struct TextSource {
 	void UpdateFont();
 	void GetStringFormat(StringFormat &format);
 	void RemoveNewlinePadding(const StringFormat &format, RectF &box);
-	void CalculateTextSizes(const StringFormat &format,
-			RectF &bounding_box, SIZE &text_size);
-	void RenderOutlineText(Graphics &graphics,
-			const GraphicsPath &path,
-			const Brush &brush);
+	void CalculateTextSizes(const StringFormat &format, RectF &bounding_box, SIZE &text_size);
+	void RenderOutlineText(Graphics &graphics, const GraphicsPath &path, const Brush &brush);
 	void RenderText();
 	void LoadFileText();
+	void TransformText();
+	void SetAntiAliasing(Graphics &graphics_bitmap);
 
 	const char *GetMainString(const char *str);
 
 	inline void Update(obs_data_t *settings);
 	inline void Tick(float seconds);
-	inline void Render(gs_effect_t *effect);
+	inline void Render();
 };
 
 static time_t get_modified_timestamp(const char *filename)
@@ -307,12 +335,13 @@ void TextSource::UpdateFont()
 
 void TextSource::GetStringFormat(StringFormat &format)
 {
-	UINT flags = StringFormatFlagsNoFitBlackBox |
-		StringFormatFlagsMeasureTrailingSpaces;
+	UINT flags = StringFormatFlagsNoFitBlackBox;
+
+	if (!use_extents || old_extents)
+		flags |= StringFormatFlagsMeasureTrailingSpaces;
 
 	if (vertical)
-		flags |= StringFormatFlagsDirectionVertical |
-			StringFormatFlagsDirectionRightToLeft;
+		flags |= StringFormatFlagsDirectionVertical | StringFormatFlagsDirectionRightToLeft;
 
 	format.SetFormatFlags(flags);
 	format.SetTrimming(StringTrimmingWord);
@@ -368,12 +397,10 @@ void TextSource::RemoveNewlinePadding(const StringFormat &format, RectF &box)
 	RectF after;
 	Status stat;
 
-	stat = graphics.MeasureString(L"W", 2, font.get(), PointF(0.0f, 0.0f),
-			&format, &before);
+	stat = graphics.MeasureString(L"W", 2, font.get(), PointF(0.0f, 0.0f), &format, &before);
 	warn_stat("MeasureString (without newline)");
 
-	stat = graphics.MeasureString(L"W\n", 3, font.get(), PointF(0.0f, 0.0f),
-			&format, &after);
+	stat = graphics.MeasureString(L"W\n", 3, font.get(), PointF(0.0f, 0.0f), &format, &after);
 	warn_stat("MeasureString (with newline)");
 
 	float offset_cx = after.Width - before.Width;
@@ -397,12 +424,11 @@ void TextSource::RemoveNewlinePadding(const StringFormat &format, RectF &box)
 			box.X -= offset_cx;
 	}
 
-	box.Width  -= offset_cx;
+	box.Width -= offset_cx;
 	box.Height -= offset_cy;
 }
 
-void TextSource::CalculateTextSizes(const StringFormat &format,
-		RectF &bounding_box, SIZE &text_size)
+void TextSource::CalculateTextSizes(const StringFormat &format, RectF &bounding_box, SIZE &text_size)
 {
 	RectF layout_box;
 	RectF temp_box;
@@ -411,26 +437,22 @@ void TextSource::CalculateTextSizes(const StringFormat &format,
 	if (!text.empty()) {
 		if (use_extents && wrap) {
 			layout_box.X = layout_box.Y = 0;
-			layout_box.Width  = float(extents_cx);
+			layout_box.Width = float(extents_cx);
 			layout_box.Height = float(extents_cy);
 
 			if (use_outline) {
-				layout_box.Width  -= outline_size;
+				layout_box.Width -= outline_size;
 				layout_box.Height -= outline_size;
 			}
 
-			stat = graphics.MeasureString(text.c_str(),
-					(int)text.size() + 1, font.get(),
-					layout_box, &format,
-					&bounding_box);
+			stat = graphics.MeasureString(text.c_str(), (int)text.size() + 1, font.get(), layout_box,
+						      &format, &bounding_box);
 			warn_stat("MeasureString (wrapped)");
 
 			temp_box = bounding_box;
 		} else {
-			stat = graphics.MeasureString(text.c_str(),
-					(int)text.size() + 1, font.get(),
-					PointF(0.0f, 0.0f), &format,
-					&bounding_box);
+			stat = graphics.MeasureString(text.c_str(), (int)text.size() + 1, font.get(),
+						      PointF(0.0f, 0.0f), &format, &bounding_box);
 			warn_stat("MeasureString (non-wrapped)");
 
 			temp_box = bounding_box;
@@ -441,7 +463,7 @@ void TextSource::CalculateTextSizes(const StringFormat &format,
 			RemoveNewlinePadding(format, bounding_box);
 
 			if (use_outline) {
-				bounding_box.Width  += outline_size;
+				bounding_box.Width += outline_size;
 				bounding_box.Height += outline_size;
 			}
 		}
@@ -470,6 +492,20 @@ void TextSource::CalculateTextSizes(const StringFormat &format,
 	if (use_extents) {
 		text_size.cx = extents_cx;
 		text_size.cy = extents_cy;
+
+		if (!old_extents && !wrap) {
+			if (align == Align::Center) {
+				bounding_box.X += (extents_cx - bounding_box.Width) / 2;
+			} else if (align == Align::Right) {
+				bounding_box.X += extents_cx - bounding_box.Width;
+			}
+
+			if (valign == VAlign::Center) {
+				bounding_box.Y += (extents_cy - bounding_box.Height) / 2;
+			} else if (valign == VAlign::Bottom) {
+				bounding_box.Y += extents_cy - bounding_box.Height;
+			}
+		}
 	}
 
 	text_size.cx += text_size.cx % 2;
@@ -491,13 +527,11 @@ void TextSource::CalculateTextSizes(const StringFormat &format,
 
 	/* the internal text-rendering bounding box for is reset to
 	 * its internal value in case the texture gets cut off */
-	bounding_box.Width  = temp_box.Width;
+	bounding_box.Width = temp_box.Width;
 	bounding_box.Height = temp_box.Height;
 }
 
-void TextSource::RenderOutlineText(Graphics &graphics,
-		const GraphicsPath &path,
-		const Brush &brush)
+void TextSource::RenderOutlineText(Graphics &graphics, const GraphicsPath &path, const Brush &brush)
 {
 	DWORD outline_rgba = calc_color(outline_color, outline_opacity);
 	Status stat;
@@ -524,15 +558,12 @@ void TextSource::RenderText()
 	GetStringFormat(format);
 	CalculateTextSizes(format, box, size);
 
-	unique_ptr<uint8_t> bits(new uint8_t[size.cx * size.cy * 4]);
-	Bitmap bitmap(size.cx, size.cy, 4 * size.cx, PixelFormat32bppARGB,
-			bits.get());
+	unique_ptr<uint8_t[]> bits(new uint8_t[size.cx * size.cy * 4]);
+	Bitmap bitmap(size.cx, size.cy, 4 * size.cx, PixelFormat32bppARGB, bits.get());
 
 	Graphics graphics_bitmap(&bitmap);
-	LinearGradientBrush brush(RectF(0, 0, (float)size.cx, (float)size.cy),
-			Color(calc_color(color, opacity)),
-			Color(calc_color(color2, opacity2)),
-			gradient_dir, 1);
+	LinearGradientBrush brush(RectF(0, 0, (float)size.cx, (float)size.cy), Color(calc_color(color, opacity)),
+				  Color(calc_color(color2, opacity2)), gradient_dir, 1);
 	DWORD full_bk_color = bk_color & 0xFFFFFF;
 
 	if (!text.empty() || use_extents)
@@ -550,9 +581,8 @@ void TextSource::RenderText()
 		warn_stat("graphics_bitmap.Clear");
 	}
 
-	graphics_bitmap.SetTextRenderingHint(TextRenderingHintAntiAlias);
 	graphics_bitmap.SetCompositingMode(CompositingModeSourceOver);
-	graphics_bitmap.SetSmoothingMode(SmoothingModeAntiAlias);
+	SetAntiAliasing(graphics_bitmap);
 
 	if (!text.empty()) {
 		if (use_outline) {
@@ -562,16 +592,14 @@ void TextSource::RenderText()
 			GraphicsPath path;
 
 			font->GetFamily(&family);
-			stat = path.AddString(text.c_str(), (int)text.size(),
-					&family, font->GetStyle(),
-					font->GetSize(), box, &format);
+			stat = path.AddString(text.c_str(), (int)text.size(), &family, font->GetStyle(),
+					      font->GetSize(), box, &format);
 			warn_stat("path.AddString");
 
 			RenderOutlineText(graphics_bitmap, path, brush);
 		} else {
-			stat = graphics_bitmap.DrawString(text.c_str(),
-					(int)text.size(), font.get(),
-					box, &format, &brush);
+			stat = graphics_bitmap.DrawString(text.c_str(), (int)text.size(), font.get(), box, &format,
+							  &brush);
 			warn_stat("graphics_bitmap.DrawString");
 		}
 	}
@@ -581,9 +609,8 @@ void TextSource::RenderText()
 		if (tex)
 			gs_texture_destroy(tex);
 
-		const uint8_t *data = (uint8_t*)bits.get();
-		tex = gs_texture_create(size.cx, size.cy, GS_BGRA, 1, &data,
-				GS_DYNAMIC);
+		const uint8_t *data = (uint8_t *)bits.get();
+		tex = gs_texture_create(size.cx, size.cy, GS_BGRA, 1, &data, GS_DYNAMIC);
 
 		obs_leave_graphics();
 
@@ -611,7 +638,7 @@ const char *TextSource::GetMainString(const char *str)
 
 	const char *temp = str + len;
 
-	while(temp != str) {
+	while (temp != str) {
 		temp--;
 
 		if (temp[0] == '\n' && temp[1] != 0) {
@@ -632,55 +659,90 @@ void TextSource::LoadFileText()
 		text.push_back('\n');
 }
 
-#define obs_data_get_uint32 (uint32_t)obs_data_get_int
+void TextSource::TransformText()
+{
+	const locale loc = locale(obs_get_locale());
+	const ctype<wchar_t> &f = use_facet<ctype<wchar_t>>(loc);
+	if (text_transform == S_TRANSFORM_UPPERCASE)
+		f.toupper(&text[0], &text[0] + text.size());
+	else if (text_transform == S_TRANSFORM_LOWERCASE)
+		f.tolower(&text[0], &text[0] + text.size());
+	else if (text_transform == S_TRANSFORM_STARTCASE) {
+		bool upper = true;
+		for (wstring::iterator it = text.begin(); it != text.end(); ++it) {
+			const wchar_t upper_char = f.toupper(*it);
+			const wchar_t lower_char = f.tolower(*it);
+			if (upper && lower_char != upper_char) {
+				upper = false;
+				*it = upper_char;
+			} else if (lower_char != upper_char) {
+				*it = lower_char;
+			} else {
+				upper = iswspace(*it);
+			}
+		}
+	}
+}
+
+void TextSource::SetAntiAliasing(Graphics &graphics_bitmap)
+{
+	if (!antialiasing) {
+		graphics_bitmap.SetTextRenderingHint(TextRenderingHintSingleBitPerPixel);
+		graphics_bitmap.SetSmoothingMode(SmoothingModeNone);
+		return;
+	}
+
+	graphics_bitmap.SetTextRenderingHint(TextRenderingHintAntiAlias);
+	graphics_bitmap.SetSmoothingMode(SmoothingModeAntiAlias);
+}
+
+#define obs_data_get_uint32 (uint32_t) obs_data_get_int
 
 inline void TextSource::Update(obs_data_t *s)
 {
-	const char *new_text   = obs_data_get_string(s, S_TEXT);
-	obs_data_t *font_obj   = obs_data_get_obj(s, S_FONT);
-	const char *align_str  = obs_data_get_string(s, S_ALIGN);
+	const char *new_text = obs_data_get_string(s, S_TEXT);
+	obs_data_t *font_obj = obs_data_get_obj(s, S_FONT);
+	const char *align_str = obs_data_get_string(s, S_ALIGN);
 	const char *valign_str = obs_data_get_string(s, S_VALIGN);
-	uint32_t new_color     = obs_data_get_uint32(s, S_COLOR);
-	uint32_t new_opacity   = obs_data_get_uint32(s, S_OPACITY);
-	bool gradient          = obs_data_get_bool(s, S_GRADIENT);
-	uint32_t new_color2    = obs_data_get_uint32(s, S_GRADIENT_COLOR);
-	uint32_t new_opacity2  = obs_data_get_uint32(s, S_GRADIENT_OPACITY);
-	float new_grad_dir     = (float)obs_data_get_double(s, S_GRADIENT_DIR);
-	bool new_vertical      = obs_data_get_bool(s, S_VERTICAL);
-	bool new_outline       = obs_data_get_bool(s, S_OUTLINE);
-	uint32_t new_o_color   = obs_data_get_uint32(s, S_OUTLINE_COLOR);
+	uint32_t new_color = obs_data_get_uint32(s, S_COLOR);
+	uint32_t new_opacity = obs_data_get_uint32(s, S_OPACITY);
+	bool gradient = obs_data_get_bool(s, S_GRADIENT);
+	uint32_t new_color2 = obs_data_get_uint32(s, S_GRADIENT_COLOR);
+	uint32_t new_opacity2 = obs_data_get_uint32(s, S_GRADIENT_OPACITY);
+	float new_grad_dir = (float)obs_data_get_double(s, S_GRADIENT_DIR);
+	bool new_vertical = obs_data_get_bool(s, S_VERTICAL);
+	bool new_outline = obs_data_get_bool(s, S_OUTLINE);
+	uint32_t new_o_color = obs_data_get_uint32(s, S_OUTLINE_COLOR);
 	uint32_t new_o_opacity = obs_data_get_uint32(s, S_OUTLINE_OPACITY);
-	uint32_t new_o_size    = obs_data_get_uint32(s, S_OUTLINE_SIZE);
-	bool new_use_file      = obs_data_get_bool(s, S_USE_FILE);
-	const char *new_file   = obs_data_get_string(s, S_FILE);
-	bool new_chat_mode     = obs_data_get_bool(s, S_CHATLOG_MODE);
-	int new_chat_lines     = (int)obs_data_get_int(s, S_CHATLOG_LINES);
-	bool new_extents       = obs_data_get_bool(s, S_EXTENTS);
-	bool new_extents_wrap  = obs_data_get_bool(s, S_EXTENTS_WRAP);
-	uint32_t n_extents_cx  = obs_data_get_uint32(s, S_EXTENTS_CX);
-	uint32_t n_extents_cy  = obs_data_get_uint32(s, S_EXTENTS_CY);
+	uint32_t new_o_size = obs_data_get_uint32(s, S_OUTLINE_SIZE);
+	bool new_use_file = obs_data_get_bool(s, S_USE_FILE);
+	const char *new_file = obs_data_get_string(s, S_FILE);
+	bool new_chat_mode = obs_data_get_bool(s, S_CHATLOG_MODE);
+	int new_chat_lines = (int)obs_data_get_int(s, S_CHATLOG_LINES);
+	bool new_extents = obs_data_get_bool(s, S_EXTENTS);
+	bool new_extents_wrap = obs_data_get_bool(s, S_EXTENTS_WRAP);
+	uint32_t n_extents_cx = obs_data_get_uint32(s, S_EXTENTS_CX);
+	uint32_t n_extents_cy = obs_data_get_uint32(s, S_EXTENTS_CY);
+	int new_text_transform = (int)obs_data_get_int(s, S_TRANSFORM);
+	bool new_antialiasing = obs_data_get_bool(s, S_ANTIALIASING);
 
-	const char *font_face  = obs_data_get_string(font_obj, "face");
-	int font_size          = (int)obs_data_get_int(font_obj, "size");
-	int64_t font_flags     = obs_data_get_int(font_obj, "flags");
-	bool new_bold          = (font_flags & OBS_FONT_BOLD) != 0;
-	bool new_italic        = (font_flags & OBS_FONT_ITALIC) != 0;
-	bool new_underline     = (font_flags & OBS_FONT_UNDERLINE) != 0;
-	bool new_strikeout     = (font_flags & OBS_FONT_STRIKEOUT) != 0;
+	const char *font_face = obs_data_get_string(font_obj, "face");
+	int font_size = (int)obs_data_get_int(font_obj, "size");
+	int64_t font_flags = obs_data_get_int(font_obj, "flags");
+	bool new_bold = (font_flags & OBS_FONT_BOLD) != 0;
+	bool new_italic = (font_flags & OBS_FONT_ITALIC) != 0;
+	bool new_underline = (font_flags & OBS_FONT_UNDERLINE) != 0;
+	bool new_strikeout = (font_flags & OBS_FONT_STRIKEOUT) != 0;
 
-	uint32_t new_bk_color   = obs_data_get_uint32(s, S_BKCOLOR);
+	uint32_t new_bk_color = obs_data_get_uint32(s, S_BKCOLOR);
 	uint32_t new_bk_opacity = obs_data_get_uint32(s, S_BKOPACITY);
 
 	/* ----------------------------- */
 
 	wstring new_face = to_wide(font_face);
 
-	if (new_face      != face      ||
-	    face_size     != font_size ||
-	    new_bold      != bold      ||
-	    new_italic    != italic    ||
-	    new_underline != underline ||
-	    new_strikeout != strikeout) {
+	if (new_face != face || face_size != font_size || new_bold != bold || new_italic != italic ||
+	    new_underline != underline || new_strikeout != strikeout) {
 
 		face = new_face;
 		face_size = font_size;
@@ -712,6 +774,8 @@ inline void TextSource::Update(obs_data_t *s)
 	wrap = new_extents_wrap;
 	extents_cx = n_extents_cx;
 	extents_cy = n_extents_cy;
+	text_transform = new_text_transform;
+	antialiasing = new_antialiasing;
 
 	if (!gradient) {
 		color2 = color;
@@ -737,6 +801,7 @@ inline void TextSource::Update(obs_data_t *s)
 		if (!text.empty())
 			text.push_back('\n');
 	}
+	TransformText();
 
 	use_outline = new_outline;
 	outline_color = new_o_color;
@@ -778,6 +843,7 @@ inline void TextSource::Tick(float seconds)
 
 		if (update_file) {
 			LoadFileText();
+			TransformText();
 			RenderText();
 			update_file = false;
 		}
@@ -789,13 +855,27 @@ inline void TextSource::Tick(float seconds)
 	}
 }
 
-inline void TextSource::Render(gs_effect_t *effect)
+inline void TextSource::Render()
 {
 	if (!tex)
 		return;
 
-	gs_effect_set_texture(gs_effect_get_param_by_name(effect, "image"), tex);
+	gs_effect_t *effect = obs_get_base_effect(OBS_EFFECT_DEFAULT);
+	gs_technique_t *tech = gs_effect_get_technique(effect, "Draw");
+
+	const bool previous = gs_framebuffer_srgb_enabled();
+	gs_enable_framebuffer_srgb(true);
+
+	gs_technique_begin(tech);
+	gs_technique_begin_pass(tech, 0);
+
+	gs_effect_set_texture_srgb(gs_effect_get_param_by_name(effect, "image"), tex);
 	gs_draw_sprite(tex, 0, cx, cy);
+
+	gs_technique_end_pass(tech);
+	gs_technique_end(tech);
+
+	gs_enable_framebuffer_srgb(previous);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -804,15 +884,18 @@ static ULONG_PTR gdip_token = 0;
 
 OBS_DECLARE_MODULE()
 OBS_MODULE_USE_DEFAULT_LOCALE("obs-text", "en-US")
+MODULE_EXPORT const char *obs_module_description(void)
+{
+	return "Windows GDI+ text source";
+}
 
-#define set_vis(var, val, show) \
-	do { \
-		p = obs_properties_get(props, val); \
+#define set_vis(var, val, show)                           \
+	do {                                              \
+		p = obs_properties_get(props, val);       \
 		obs_property_set_visible(p, var == show); \
 	} while (false)
 
-static bool use_file_changed(obs_properties_t *props, obs_property_t *p,
-		obs_data_t *s)
+static bool use_file_changed(obs_properties_t *props, obs_property_t *p, obs_data_t *s)
 {
 	bool use_file = obs_data_get_bool(s, S_USE_FILE);
 
@@ -821,8 +904,7 @@ static bool use_file_changed(obs_properties_t *props, obs_property_t *p,
 	return true;
 }
 
-static bool outline_changed(obs_properties_t *props, obs_property_t *p,
-		obs_data_t *s)
+static bool outline_changed(obs_properties_t *props, obs_property_t *p, obs_data_t *s)
 {
 	bool outline = obs_data_get_bool(s, S_OUTLINE);
 
@@ -832,8 +914,7 @@ static bool outline_changed(obs_properties_t *props, obs_property_t *p,
 	return true;
 }
 
-static bool chatlog_mode_changed(obs_properties_t *props, obs_property_t *p,
-		obs_data_t *s)
+static bool chatlog_mode_changed(obs_properties_t *props, obs_property_t *p, obs_data_t *s)
 {
 	bool chatlog_mode = obs_data_get_bool(s, S_CHATLOG_MODE);
 
@@ -841,8 +922,7 @@ static bool chatlog_mode_changed(obs_properties_t *props, obs_property_t *p,
 	return true;
 }
 
-static bool gradient_changed(obs_properties_t *props, obs_property_t *p,
-		obs_data_t *s)
+static bool gradient_changed(obs_properties_t *props, obs_property_t *p, obs_data_t *s)
 {
 	bool gradient = obs_data_get_bool(s, S_GRADIENT);
 
@@ -852,8 +932,7 @@ static bool gradient_changed(obs_properties_t *props, obs_property_t *p,
 	return true;
 }
 
-static bool extents_modified(obs_properties_t *props, obs_property_t *p,
-		obs_data_t *s)
+static bool extents_modified(obs_properties_t *props, obs_property_t *p, obs_data_t *s)
 {
 	bool use_extents = obs_data_get_bool(s, S_EXTENTS);
 
@@ -867,7 +946,7 @@ static bool extents_modified(obs_properties_t *props, obs_property_t *p,
 
 static obs_properties_t *get_properties(void *data)
 {
-	TextSource *s = reinterpret_cast<TextSource*>(data);
+	TextSource *s = reinterpret_cast<TextSource *>(data);
 	string path;
 
 	obs_properties_t *props = obs_properties_create();
@@ -895,35 +974,41 @@ static obs_properties_t *get_properties(void *data)
 	}
 
 	obs_properties_add_text(props, S_TEXT, T_TEXT, OBS_TEXT_MULTILINE);
-	obs_properties_add_path(props, S_FILE, T_FILE, OBS_PATH_FILE,
-			filter.c_str(), path.c_str());
+	obs_properties_add_path(props, S_FILE, T_FILE, OBS_PATH_FILE, filter.c_str(), path.c_str());
+
+	obs_properties_add_bool(props, S_ANTIALIASING, T_ANTIALIASING);
+
+	p = obs_properties_add_list(props, S_TRANSFORM, T_TRANSFORM, OBS_COMBO_TYPE_LIST, OBS_COMBO_FORMAT_INT);
+	obs_property_list_add_int(p, T_TRANSFORM_NONE, S_TRANSFORM_NONE);
+	obs_property_list_add_int(p, T_TRANSFORM_UPPERCASE, S_TRANSFORM_UPPERCASE);
+	obs_property_list_add_int(p, T_TRANSFORM_LOWERCASE, S_TRANSFORM_LOWERCASE);
+	obs_property_list_add_int(p, T_TRANSFORM_STARTCASE, S_TRANSFORM_STARTCASE);
 
 	obs_properties_add_bool(props, S_VERTICAL, T_VERTICAL);
+
 	obs_properties_add_color(props, S_COLOR, T_COLOR);
-	obs_properties_add_int_slider(props, S_OPACITY, T_OPACITY, 0, 100, 1);
+	p = obs_properties_add_int_slider(props, S_OPACITY, T_OPACITY, 0, 100, 1);
+	obs_property_int_set_suffix(p, "%");
 
 	p = obs_properties_add_bool(props, S_GRADIENT, T_GRADIENT);
 	obs_property_set_modified_callback(p, gradient_changed);
 
 	obs_properties_add_color(props, S_GRADIENT_COLOR, T_GRADIENT_COLOR);
-	obs_properties_add_int_slider(props, S_GRADIENT_OPACITY,
-			T_GRADIENT_OPACITY, 0, 100, 1);
-	obs_properties_add_float_slider(props, S_GRADIENT_DIR,
-			T_GRADIENT_DIR, 0, 360, 0.1);
-	
+	p = obs_properties_add_int_slider(props, S_GRADIENT_OPACITY, T_GRADIENT_OPACITY, 0, 100, 1);
+	obs_property_int_set_suffix(p, "%");
+	obs_properties_add_float_slider(props, S_GRADIENT_DIR, T_GRADIENT_DIR, 0, 360, 0.1);
+
 	obs_properties_add_color(props, S_BKCOLOR, T_BKCOLOR);
-	obs_properties_add_int_slider(props, S_BKOPACITY, T_BKOPACITY,
-			0, 100, 1);
+	p = obs_properties_add_int_slider(props, S_BKOPACITY, T_BKOPACITY, 0, 100, 1);
+	obs_property_int_set_suffix(p, "%");
 
-	p = obs_properties_add_list(props, S_ALIGN, T_ALIGN,
-			OBS_COMBO_TYPE_LIST, OBS_COMBO_FORMAT_STRING);
-	obs_property_list_add_string(p, T_ALIGN_LEFT,   S_ALIGN_LEFT);
+	p = obs_properties_add_list(props, S_ALIGN, T_ALIGN, OBS_COMBO_TYPE_LIST, OBS_COMBO_FORMAT_STRING);
+	obs_property_list_add_string(p, T_ALIGN_LEFT, S_ALIGN_LEFT);
 	obs_property_list_add_string(p, T_ALIGN_CENTER, S_ALIGN_CENTER);
-	obs_property_list_add_string(p, T_ALIGN_RIGHT,  S_ALIGN_RIGHT);
+	obs_property_list_add_string(p, T_ALIGN_RIGHT, S_ALIGN_RIGHT);
 
-	p = obs_properties_add_list(props, S_VALIGN, T_VALIGN,
-			OBS_COMBO_TYPE_LIST, OBS_COMBO_FORMAT_STRING);
-	obs_property_list_add_string(p, T_VALIGN_TOP,    S_VALIGN_TOP);
+	p = obs_properties_add_list(props, S_VALIGN, T_VALIGN, OBS_COMBO_TYPE_LIST, OBS_COMBO_FORMAT_STRING);
+	obs_property_list_add_string(p, T_VALIGN_TOP, S_VALIGN_TOP);
 	obs_property_list_add_string(p, T_VALIGN_CENTER, S_VALIGN_CENTER);
 	obs_property_list_add_string(p, T_VALIGN_BOTTOM, S_VALIGN_BOTTOM);
 
@@ -932,14 +1017,13 @@ static obs_properties_t *get_properties(void *data)
 
 	obs_properties_add_int(props, S_OUTLINE_SIZE, T_OUTLINE_SIZE, 1, 20, 1);
 	obs_properties_add_color(props, S_OUTLINE_COLOR, T_OUTLINE_COLOR);
-	obs_properties_add_int_slider(props, S_OUTLINE_OPACITY,
-			T_OUTLINE_OPACITY, 0, 100, 1);
+	p = obs_properties_add_int_slider(props, S_OUTLINE_OPACITY, T_OUTLINE_OPACITY, 0, 100, 1);
+	obs_property_int_set_suffix(p, "%");
 
 	p = obs_properties_add_bool(props, S_CHATLOG_MODE, T_CHATLOG_MODE);
 	obs_property_set_modified_callback(p, chatlog_mode_changed);
 
-	obs_properties_add_int(props, S_CHATLOG_LINES, T_CHATLOG_LINES,
-			1, 1000, 1);
+	obs_properties_add_int(props, S_CHATLOG_LINES, T_CHATLOG_LINES, 1, 1000, 1);
 
 	p = obs_properties_add_bool(props, S_EXTENTS, T_EXTENTS);
 	obs_property_set_modified_callback(p, extents_modified);
@@ -951,74 +1035,124 @@ static obs_properties_t *get_properties(void *data)
 	return props;
 }
 
+static void defaults(obs_data_t *settings, int ver)
+{
+	obs_data_t *font_obj = obs_data_create();
+	obs_data_set_default_string(font_obj, "face", "Arial");
+	obs_data_set_default_int(font_obj, "size", ver == 1 ? 36 : 256);
+
+	obs_data_set_default_obj(settings, S_FONT, font_obj);
+	obs_data_set_default_string(settings, S_ALIGN, S_ALIGN_LEFT);
+	obs_data_set_default_string(settings, S_VALIGN, S_VALIGN_TOP);
+	obs_data_set_default_int(settings, S_COLOR, 0xFFFFFF);
+	obs_data_set_default_int(settings, S_OPACITY, 100);
+	obs_data_set_default_int(settings, S_GRADIENT_COLOR, 0xFFFFFF);
+	obs_data_set_default_int(settings, S_GRADIENT_OPACITY, 100);
+	obs_data_set_default_double(settings, S_GRADIENT_DIR, 90.0);
+	obs_data_set_default_int(settings, S_BKCOLOR, 0x000000);
+	obs_data_set_default_int(settings, S_BKOPACITY, 0);
+	obs_data_set_default_int(settings, S_OUTLINE_SIZE, 2);
+	obs_data_set_default_int(settings, S_OUTLINE_COLOR, 0xFFFFFF);
+	obs_data_set_default_int(settings, S_OUTLINE_OPACITY, 100);
+	obs_data_set_default_int(settings, S_CHATLOG_LINES, 6);
+	obs_data_set_default_bool(settings, S_EXTENTS_WRAP, true);
+	obs_data_set_default_int(settings, S_EXTENTS_CX, 100);
+	obs_data_set_default_int(settings, S_EXTENTS_CY, 100);
+	obs_data_set_default_int(settings, S_TRANSFORM, S_TRANSFORM_NONE);
+	obs_data_set_default_bool(settings, S_ANTIALIASING, true);
+
+	obs_data_release(font_obj);
+};
+
+static void missing_file_callback(void *src, const char *new_path, void *data)
+{
+	TextSource *s = reinterpret_cast<TextSource *>(src);
+
+	obs_source_t *source = s->source;
+	obs_data_t *settings = obs_source_get_settings(source);
+	obs_data_set_string(settings, S_FILE, new_path);
+	obs_source_update(source, settings);
+	obs_data_release(settings);
+
+	UNUSED_PARAMETER(data);
+}
+
 bool obs_module_load(void)
 {
 	obs_source_info si = {};
 	si.id = "text_gdiplus";
 	si.type = OBS_SOURCE_TYPE_INPUT;
-	si.output_flags = OBS_SOURCE_VIDEO;
+	si.output_flags = OBS_SOURCE_VIDEO | OBS_SOURCE_CUSTOM_DRAW | OBS_SOURCE_CAP_OBSOLETE | OBS_SOURCE_SRGB;
 	si.get_properties = get_properties;
+	si.icon_type = OBS_ICON_TYPE_TEXT;
 
-	si.get_name = [] (void*)
-	{
+	si.get_name = [](void *) {
 		return obs_module_text("TextGDIPlus");
 	};
-	si.create = [] (obs_data_t *settings, obs_source_t *source)
-	{
-		return (void*)new TextSource(source, settings);
+	si.create = [](obs_data_t *settings, obs_source_t *source) {
+		return (void *)new TextSource(source, settings, true);
 	};
-	si.destroy = [] (void *data)
-	{
-		delete reinterpret_cast<TextSource*>(data);
+	si.destroy = [](void *data) {
+		delete reinterpret_cast<TextSource *>(data);
 	};
-	si.get_width = [] (void *data)
-	{
-		return reinterpret_cast<TextSource*>(data)->cx;
+	si.get_width = [](void *data) {
+		return reinterpret_cast<TextSource *>(data)->cx;
 	};
-	si.get_height = [] (void *data)
-	{
-		return reinterpret_cast<TextSource*>(data)->cy;
+	si.get_height = [](void *data) {
+		return reinterpret_cast<TextSource *>(data)->cy;
 	};
-	si.get_defaults = [] (obs_data_t *settings)
-	{
-		obs_data_t *font_obj = obs_data_create();
-		obs_data_set_default_string(font_obj, "face", "Arial");
-		obs_data_set_default_int(font_obj, "size", 36);
+	si.get_defaults = [](obs_data_t *settings) {
+		defaults(settings, 1);
+	};
+	si.update = [](void *data, obs_data_t *settings) {
+		reinterpret_cast<TextSource *>(data)->Update(settings);
+	};
+	si.video_tick = [](void *data, float seconds) {
+		reinterpret_cast<TextSource *>(data)->Tick(seconds);
+	};
+	si.video_render = [](void *data, gs_effect_t *) {
+		reinterpret_cast<TextSource *>(data)->Render();
+	};
+	si.missing_files = [](void *data) {
+		TextSource *s = reinterpret_cast<TextSource *>(data);
+		obs_missing_files_t *files = obs_missing_files_create();
 
-		obs_data_set_default_obj(settings, S_FONT, font_obj);
-		obs_data_set_default_string(settings, S_ALIGN, S_ALIGN_LEFT);
-		obs_data_set_default_string(settings, S_VALIGN, S_VALIGN_TOP);
-		obs_data_set_default_int(settings, S_COLOR, 0xFFFFFF);
-		obs_data_set_default_int(settings, S_OPACITY, 100);
-		obs_data_set_default_int(settings, S_GRADIENT_COLOR, 0xFFFFFF);
-		obs_data_set_default_int(settings, S_GRADIENT_OPACITY, 100);
-		obs_data_set_default_double(settings, S_GRADIENT_DIR, 90.0);
-		obs_data_set_default_int(settings, S_BKCOLOR, 0x000000);
-		obs_data_set_default_int(settings, S_BKOPACITY, 0);
-		obs_data_set_default_int(settings, S_OUTLINE_SIZE, 2);
-		obs_data_set_default_int(settings, S_OUTLINE_COLOR, 0xFFFFFF);
-		obs_data_set_default_int(settings, S_OUTLINE_OPACITY, 100);
-		obs_data_set_default_int(settings, S_CHATLOG_LINES, 6);
-		obs_data_set_default_bool(settings, S_EXTENTS_WRAP, true);
-		obs_data_set_default_int(settings, S_EXTENTS_CX, 100);
-		obs_data_set_default_int(settings, S_EXTENTS_CY, 100);
+		obs_source_t *source = s->source;
+		obs_data_t *settings = obs_source_get_settings(source);
 
-		obs_data_release(font_obj);
+		bool read = obs_data_get_bool(settings, S_USE_FILE);
+		const char *path = obs_data_get_string(settings, S_FILE);
+
+		if (read && strcmp(path, "") != 0) {
+			if (!os_file_exists(path)) {
+				obs_missing_file_t *file = obs_missing_file_create(
+					path, missing_file_callback, OBS_MISSING_FILE_SOURCE, s->source, NULL);
+
+				obs_missing_files_add_file(files, file);
+			}
+		}
+
+		obs_data_release(settings);
+
+		return files;
 	};
-	si.update = [] (void *data, obs_data_t *settings)
-	{
-		reinterpret_cast<TextSource*>(data)->Update(settings);
+
+	obs_source_info si_v2 = si;
+	si_v2.version = 2;
+	si_v2.get_defaults = [](obs_data_t *settings) {
+		defaults(settings, 2);
 	};
-	si.video_tick = [] (void *data, float seconds)
-	{
-		reinterpret_cast<TextSource*>(data)->Tick(seconds);
-	};
-	si.video_render = [] (void *data, gs_effect_t *effect)
-	{
-		reinterpret_cast<TextSource*>(data)->Render(effect);
+
+	obs_source_info si_v3 = si_v2;
+	si_v3.version = 3;
+	si_v3.output_flags &= ~OBS_SOURCE_CAP_OBSOLETE;
+	si_v3.create = [](obs_data_t *settings, obs_source_t *source) {
+		return (void *)new TextSource(source, settings, false);
 	};
 
 	obs_register_source(&si);
+	obs_register_source(&si_v2);
+	obs_register_source(&si_v3);
 
 	const GdiplusStartupInput gdip_input;
 	GdiplusStartup(&gdip_token, &gdip_input, nullptr);
